
library(ape)
library(BAMMtools)
library(openxlsx)

source("color.clade.R")

check_and_fix_ultrametric <- function(phy){
	
	if (!is.ultrametric(phy)){
		
		vv <- vcv.phylo(phy)
		dx <- diag(vv)
		mxx <- max(dx) - dx
		for (i in 1:length(mxx)){
			phy$edge.length[phy$edge[,2] == i] <- phy$edge.length[phy$edge[,2] == i] + mxx[i]
		}
		if (!is.ultrametric(phy)){
			stop("Ultrametric fix failed\n")
		}	
	}
	
	return(phy)
}

# ------------------------


v <- read.tree("data/actinopt_12k_treePL.tre")
v <- check_and_fix_ultrametric(v)

xx <- read.csv("data/marine-fw-rates-june2018.csv", stringsAsFactors=F)
rownames(xx) <- xx$treename

#----- State scoring
# categories
#   Fw
#   Fw-Ma
#   Ma
#   Br-Ma
#   Br-Fw
#   Br-Fw-Ma
#   Br

salinity <- character(nrow(xx))
salinity[xx$Freshwater == 1 & xx$Marine == 0 & xx$Brackish == 0] <- "Fw"
salinity[xx$Freshwater == 1 & xx$Marine == 1 & xx$Brackish == 1] <- "Br-Fw-Ma"
salinity[xx$Freshwater == 1 & xx$Marine == 1 & xx$Brackish == 0] <- "Fw-Ma"
salinity[xx$Freshwater == 0 & xx$Marine == 1 & xx$Brackish == 0] <- "Ma"
salinity[xx$Freshwater == 0 & xx$Marine == 0 & xx$Brackish == 1] <- "Br"
salinity[xx$Freshwater == 0 & xx$Marine == 1 & xx$Brackish == 1] <- "Br-Ma"
salinity[xx$Freshwater == 1 & xx$Marine == 0 & xx$Brackish == 1] <- "Br-Fw"

xx$salinity <- salinity

# =========================================== #

cladedata <- read.xlsx("data/Appendix-S1.xlsx")
cumsum(cladedata$fw_total) / sum(cladedata$fw_total)

cladedata[1:22, 1:5]

radiations <- grep("Yes", cladedata$radiation_in_FW, ignore.case = T)


#=============================================# 

#xx <- polar.phylo(v, lwd=2)
#colorClade(v, xx$segs, "Drymarchon_corais", "Epictia_diaplocia", col="gray50", border=1, alpha = 0.4)
#colorClade(v, xx$segs, "Atractus_badius", "Erythrolamprus_cobella", col="green", border="1", alpha = 0.4)
#colorClade(v, xx$segs, "Drymarchon_corais", "Epictia_diaplocia", col="gray50", border=1, alpha = 0.4)
#colorClade(v, xx$segs, "Drymarchon_corais", "Dendrophidion_dendrophis", col=2, border=1, alpha = 0.4)

 

# Plot only groups with 50 or more FW species 
y <- cladedata[cladedata$fw_total > 50, ]



 
### Get all tips from these groups from full tree:
tiplist <- list()
for (ii in 1:nrow(y)){
	tips <- c(y$tree_tip1[ii], y$tree_tip2[ii])
	tiplist[[ii]] <- extract.clade(v, node = getMRCA(v, tip = tips))$tip.label
}

drops <- sample(v$tip.label, size = 8000)
vsub <- drop.tip(v, tip = drops)

mrca_nodes <- numeric(nrow(y))

for (ii in 1:nrow(y)){
	iset <- intersect(tiplist[[ii]], vsub$tip.label)
	mrca_nodes[ii] <- getMRCA(vsub, tip = iset)
}

 

quartz.options(height = 5, width=5)
par(oma=c(1,1,1,1), mar=c(0,0,0,0))

z <- polar.phylo(vsub, lwd=0.6 )

#tipset <- extract.clade(v, node = nd)$tip.label
#indices <- match(tipset, v$tip.label)

## Percomorpha:
#tips <- extract.clade(v, node = getMRCA(v, c("Chilara_taylori", "Dissostichus_mawsoni")))$tip.label
#tips <- intersect(tips, vsub$tip.label)
#nd <- getMRCA(vsub, tips)
#colorClade(vsub, z$segs, node_mrca = nd, col="gray80", border=1, alpha = 0.8)



for (ii in 1:length(mrca_nodes)){
	if (y$subclade[ii] != "Gobioidei")
		colorClade(vsub, z$segs, node_mrca = mrca_nodes[ii], col="red", border=1, alpha = 0.6)
}


mar_clades <- c("Blenniiformes", "Anguilliformes", "Labriformes", "Tetraodontiformes", "Pleuronectiformes", "Syngnathiformes", "Stomiatiformes")
mrca_nodes_sea <- numeric(length(mar_clades))

for (ii in 1:length(mar_clades)){
	tmp <- xx$tip[xx$order == mar_clades[ii]]
	tmp <- intersect(tmp, vsub$tip.label) 
	mrca_nodes_sea[ii] <- getMRCA(vsub, tip = tmp)
	
	colorClade(vsub, z$segs, node_mrca = mrca_nodes_sea[ii], col="blue", border=1, alpha = 0.6)
	
}
  
colorClade(vsub, z$segs, node_mrca = mrca_nodes[which(y$subclade == "Gobioidei")], col="darkgreen", border=1, alpha = 0.6)
	
#=================================================== #
# Figure 2
 
is_radiation <- cladedata$radiation_in_FW == "Yes" 

col <- rep("black", nrow(cladedata))
cex <- rep(1.3, nrow(cladedata))
col[!is_radiation] <- "gray80"
cex[!is_radiation] <- 0.8



## 

cs <- cumsum(cladedata$fw_total)

quartz.options(height=5, width=10)
plot.new()
par(oma=c(1,1,1,1), mfrow = c(1,2), mar=c(6,6,1,1))

plot.new()

plot.window(xlim=c(0,53), ylim=c(0,10000))

# xw <- 0.5
# for (ii in 1:nrow(cladedata)){
# 	xv <- c(ii-xw, ii-xw, ii+xw, ii+xw)
# 	yv <- c(0, rep(cladedata$fw_total[ii], 2),   0)
# 	#polygon(xv, yv, border= "black", col=col[ii])
 	
# }

#lines(x=c(0,55), y=c(13800, 13800), lwd=3, col="red", lty = "dotted")

#points(1:length(cs), cs, pch=21, bg=col, cex=0.8)
points(1:nrow(cladedata), cladedata$fw_total, pch=21, bg=col, cex=0.8)


axis(1, at=seq(-10,50,by=10), cex.axis=0.8)
axis(2, at=seq(-2000, 10000, by=2000), las=1, cex.axis=0.8)
mtext(side=1, text = "Clade rank", line=2.5, cex=1)
mtext(side=2, text = "Species richness", line=3, cex=1)
mtext(text = "(a)", side=2, at = 10000, line=3.8, cex=1.3, las=1)


plot.new() 
plot.window(xlim=c(0,4), ylim=c(0,4), asp=1)

r_other <- cladedata$richness_total - cladedata$fw_total + 1

abline(0,1,col="gray60", lwd=3, lty="dotted")

points(log10(r_other), log10(cladedata$fw_total), cex=cex, pch=21, bg=col)

axis(1, at=seq(-1,4,by=1), labels=c(NA, 1, 10, 100, 1000, 10000), las=1, cex.axis=0.8)
axis(2, at=seq(-1,4,by=1), labels=c(NA, 1, 10, 100, 1000, 10000), las=1, cex.axis=0.8)

mtext(side=1, text = "Marine / brackish species", line=2.5, cex=1)
mtext(side=2, text = "Freshwater-only species", line=3, cex=1)
mtext(text = "(b)", side=2, at = 4, line=3.8, cex=1.3, las=1)


#=================================== # 
# Figure 3
# Rate quantile plot

macol <- "cornflowerblue"
fwcol <- "darkorange2"
xx    <- xx[!is.na(xx$tv.lambda), ]  #Genetic data only taxa...

isFW <- xx$Freshwater == 1 & xx$Marine == 0  
isMA <- xx$Freshwater == 0 & xx$Marine == 1  

fwlam <- xx$tv.lambda[!is.na(xx$tv.lambda) & isFW]
fwdr  <- xx$dr.mean[!is.na(xx$dr.mean) & isFW]

fwlamnc <- xx$tv.lambda[!is.na(xx$tv.lambda) & isFW & xx$order != "Cichliformes"]
fwdrnc  <- xx$dr.mean[!is.na(xx$dr.mean) & isFW & xx$order != "Cichliformes"]



malam <- xx$tv.lambda[!is.na(xx$tv.lambda) & isMA]
madr  <- xx$dr.mean[!is.na(xx$dr.mean) & isMA]

tvec <- 2^(-6:2)
qvec <- seq(0.01, 0.99, by = 0.01)

######################

quartz.options(height=5.5, width=9)
plot.new()
par(oma=c(1,1,1,1), mfcol = c(2,3), mar=c(5,5,1,1))

plot.new()
plot.window(xlim=c(-5.2, 1), ylim = c(0,1))

qq1 <- quantile(malam, qvec)
qq2 <- quantile(fwlam, qvec)
qq3 <- quantile(fwlamnc, qvec)

lines(x=log2(qq1), y=qvec, col = macol, lwd=3.5)
lines(x=log2(qq2), y=qvec, col = fwcol, lwd=2)
#lines(x=log2(qq3), y=qvec, col = fwcol, lwd=3, lty = "dotted")

xlabs <- c(NA, 0.03, 0.06, 0.12, 0.25, 0.5, 1, 2, NA)
axis(1, at=seq(-6, 2, by=1), labels=xlabs, cex.axis=0.8)
axis(2, at=seq(-0.2, 1, by=0.2), cex.axis=0.8, las=1)

mtext(side = 1, text = expression(paste("Speciation rate, ", lambda ["BAMM"])), line=2.6, cex=0.8)
mtext(side = 2, text = "Quantile of rate distribution", line = 2.5, cex=0.8)




# 
plot.new()
#plot.new()
plot.window(xlim=c(-5.2, 1), ylim = c(0,1))

qq1 <- quantile(madr, qvec)
qq2 <- quantile(fwdr, qvec)
qq3 <- quantile(fwdrnc, qvec)

lines(x=log2(qq1), y=qvec, col = macol, lwd=3.5)
lines(x=log2(qq2), y=qvec, col = fwcol, lwd=2)

axis(1, at=seq(-6, 2, by=1), labels=xlabs, cex.axis=0.8)
axis(2, at=seq(-0.2, 1, by=0.2), cex.axis=0.8, las=1)
mtext(side = 1, text = expression(paste("Speciation rate, ", lambda ["DR"])), line=2.6, cex=0.8)
mtext(side = 2, text = "Quantile of rate distribution", line = 2.5, cex=0.8)

#### Density plot
bksvec <- 2^seq(-7, 3, by=0.125)

h1 <- hist(fwlam, breaks=bksvec, plot=F)
h2 <- hist(malam, breaks=bksvec, plot=F)

d1 <- density(fwlam, bw=0.01)
d2 <- density(malam , bw=0.01)

plot.new()

plot.window(xlim=c(-6, -1), ylim = c(0,20))

lines(log2(d1$x), d1$y , col=fwcol , lwd=2)
lines(log2(d2$x), d2$y, col=macol, lwd=3)

lines(x=c(-2.75, -2.25), y=c(18, 18), lwd=2, col=fwcol) 
lines(x=c(-2.75, -2.25), y=c(16, 16), lwd=3.5, col=macol) 
text(x=-2.25, y=18, pos=4, labels="Freshwater", cex=0.85)
text(x=-2.25, y=16, pos=4, labels="Marine", cex=0.85)

axis(1, at=seq(-6, 2, by=1), labels=xlabs, cex.axis=0.8)
axis(2, at=seq(-5, 20, by=5), labels=NA)
mtext(side = 1, text = expression(paste("Speciation rate, ", lambda ["BAMM"])), line=2.6, cex=0.8)
mtext(side = 2, text = "Density", line = 1.5, cex=0.8)

# panel D ## 

 
d1 <- density(fwdr, bw=0.01)
d2 <- density(madr , bw=0.01)

plot.new()

plot.window(xlim=c(-6, -1), ylim = c(0,13))

lines(log2(d1$x), d1$y , col=fwcol , lwd=2)
lines(log2(d2$x), d2$y, col=macol, lwd=3)
 

axis(1, at=seq(-7, 2, by=1), labels=c(NA, xlabs), cex.axis=0.8)
axis(2, at=seq(-5, 20, by=5), labels=NA)
mtext(side = 1, text = expression(paste("Speciation rate, ", lambda ["DR"])), line=2.6, cex=0.8)
mtext(side = 2, text = "Density", line = 1.5, cex=0.8)
 
## PW quantile plots ##  
 
qq1 <- quantile(fwlam, seq(0.1, 0.9, by=0.1))
qq2 <- quantile(malam, seq(0.1, 0.9, by=0.1))
qq3 <- quantile(fwdr, seq(0.1, 0.9, by=0.1))
qq4 <- quantile(madr, seq(0.1, 0.9, by=0.1))


plot.new()
plot.window(xlim=c(0,0.45), ylim = c(0, 0.45), asp=1)
abline(0,1, lwd=2, col="gray60", lty="dotted")
points(qq1 ~ qq2, pch=21, col="black", bg="gray80" , cex=1)

axis(1, at=seq(-0.1, 0.5, by=0.1), cex.axis = 0.8)
axis(2, at=seq(-0.1, 0.5, by=0.1), cex.axis = 0.8, las=1)

mtext(side = 1, text = expression(paste(lambda ["BAMM"], " , marine (by quantile)")), line=2.6, cex=0.8)
mtext(side = 2, text = expression(paste(lambda ["BAMM"], " , freshwater (by quantile)")), line=2.6, cex=0.8)

text(x=qq2[9], y=qq1[9], pos=3, labels = "90%", font=3, cex=1)
text(x=qq2[8], y=qq1[8], pos=3, labels = "80%", font=3, cex=1)

### 

plot.new()
plot.window(xlim=c(0,0.45), ylim = c(0, 0.45), asp=1)
abline(0,1, lwd=2, col="gray60", lty="dotted")
points(qq3 ~ qq4, pch=21, col="black", bg="gray80" , cex=1)

axis(1, at=seq(-0.1, 0.5, by=0.1), cex.axis = 0.8)
axis(2, at=seq(-0.1, 0.5, by=0.1), cex.axis = 0.8, las=1)

text(x=qq4[9], y=qq3[9], pos=2, labels = "90%", font=3, cex=1)
text(x=qq4[8], y=qq3[8], pos=2, labels = "80%", font=3, cex=1)


mtext(side = 1, text = expression(paste(lambda ["DR"], " , marine (by quantile)")), line=2.6, cex=0.8)
mtext(side = 2, text = expression(paste(lambda ["DR"], " , freshwater (by quantile)")), line=2.6, cex=0.8)

#========================================#
# FIgure 4

library(RColorBrewer)
y <- cladedata[cladedata$include_contrast == 1, ]


# Colors
#fx <- colorRampPalette(c("blue", "lightblue", "lightblue", "coral", "red"))
fx <- colorRampPalette(c("blue", "lightblue", "coral", "red"))

colset <- fx(100)
logR <- log10(y$fw_total)

sq <- seq(1, 4, length.out=100)
cols <- rep(colset[1], length(logR))
for (i in 2:length(sq)){
	isIn <- logR > sq[i-1] & logR < sq[i]
	cols[isIn] <- colset[i]
}



colorFunction <- function(x, minn, maxx, colorset){
	
	if (min(x, na.rm=T) < minn | max(x, na.rm=T) > maxx){
		stop("minn and maxx do not span range of x")
	}
	sq <- seq(from=minn, to = maxx, length.out = length(colorset))
	cols <- rep(colorset[1], length(x))
	for (i in 2:length(sq)){
		isIn <- x >= sq[i-1] & x < sq[i]
		cols[isIn] <- colorset[i]
	}
	cols[is.na(x)] <- NA
	return(cols)
}

 
#######




ind <- 14
tips <- c(cladedata$tree_tip1[ind], cladedata$tree_tip2[ind])
xc <- extract.clade(v, getMRCA(v, tip = tips))

con_bamm <- y$lambda_bamm_fw - y$lambda_bamm_outgroup
con_dr <- y$dr_fw - y$dr_outgroup

t.test(y$lambda_bamm_fw, y$lambda_bamm_outgroup)
t.test(y$dr_fw , y$dr_outgroup)
wilcox.test(con_bamm)
wilcox.test(con_dr)

### 
bkvec <- 2^(-6:1)
labs <- c(NA, 0.03, 0.06, 0.12, 0.25, 0.5, 1, 2)

set.seed(1)
quartz.options(height=4, width=8)
plot.new()
par(oma=c(1,1,1,1), mar=c(5,5,1,1), mfrow=c(1,2))

plot.new()
plot.window(xlim=c(-5,0), ylim=c(-5,0), asp=1)

j <- function(x){
	return(jitter(x, amount=0.1))
}
lines(x=c(-6,-1.25), y=c(-6,-1.25), lwd=2, col="gray60", lty="dotted")

points(j(log2(y$lambda_bamm_outgroup)), j(log2(y$lambda_bamm_fw)), pch=21, bg=cols, cex=1.2)

axis(1, at= -6:1, labels = labs, cex.axis=0.6)
axis(2,  at = -6:1, labels = labs, cex.axis=0.6, las=1)
mtext(side = 1, text = expression(paste(lambda ["BAMM"], " , marine outgroup")), line=2.6, cex=0.9)
mtext(side = 2, text = expression(paste(lambda ["BAMM"], " , freshwater clade")), line=2.6, cex=0.9)

refcols <- colorFunction(c(1.01, 2, 3, 3.99), 1,4, colset)

ywid <- 0.35
nvec <- c(10, 100, 1000, 10000)
for (ii in 1:4){
	xp <- -0.8
	yp <- 0.35 - ii*ywid
	points(x = xp, y = yp, pch=22, bg=refcols[ii], cex=1.2)
	text(xp, yp-0.04, labels = nvec[ii], pos=4, cex= .7 , font = 3)
}

polygon(x = c(-1, -1, 0.4, 0.4), y = c(-1.3, 0.18, 0.18, -1.3), lwd=0.6, border = "gray60") 
mtext("(a)", side =2, at = 0.3, line=3, cex=1.15, las=1)

#### ---- 
plot.new()
plot.window(xlim=c(-5.2,0), ylim=c(-5.2,0), asp=1)
lines(x=c(-6,-1.25), y=c(-6,-1.25), lwd=2, col="gray60", lty="dotted")

points( log2(y$dr_outgroup), log2(y$dr_fw), pch=21, bg=cols, cex=1.2)

axis(1, at= -6:1, labels = labs, cex.axis=0.6)
axis(2,  at = -6:1, labels = labs, cex.axis=0.6, las=1)
mtext(side = 1, text = expression(paste(lambda ["DR"], " , marine outgroup")), line=2.6, cex=0.9)
mtext(side = 2, text = expression(paste(lambda ["DR"], " , freshwater clade")), line=2.6, cex=0.9)
mtext("(b)", side =2, at = 0.3, line=3, cex=1.15, las=1)

# ========================== #
 








