

set.seed(3)
# n <- 15
n <- 23
z <- rev(sort(rgeom(n, 0.003)))
z[1] <- z[1] * 7.5
z <- z / sum(z)

states <- rep(1, n)
states[sample(2:n, floor(n/2))] <- 0

rates <- runif(n, 0.02, 0.3)
rates[states == 0] <- runif(sum(states == 0), 0.02, 0.5)
rates[1] <- 0.45
 
anc_rates <- c(0.2, runif(n-1, 0.05, 0.4))
#plot(anc_rates[states==1] , rates[states==1])

dff <- data.frame(rich=z, state=states, rate = rates, anc_rate = anc_rates)

#dff <- dff[order(dff$rate), ]
#plot(dff$rate)

# rlnorm(x, meanlog, sdlog = 0.1)

rateset <- list()
for (ii in 1:nrow(dff)){
	rateset[[ii]] <- rlnorm(floor(1000*dff$rich[ii]), meanlog = log(dff$rate[ii]), sdlog=0.1)
}



mean(dff$rate[dff$state == 0])
mean(dff$rate[dff$state == 1])

z1 <- unlist(lapply(rateset, length))
z2 <- unlist(lapply(rateset, sum))
sum(z2[dff$state == 0]) / sum(z1[dff$state == 0])
sum(z2[dff$state == 1]) / sum(z1[dff$state == 1])

full_trop <- unlist(rateset[dff$state == 1])
full_temp <- unlist(rateset[dff$state == 0])

dff <- dff[order(dff$rich, decreasing=T), ]

## ------------ ##

#col_trop <- "darkorange2"
#col_temp <- "cornflowerblue"
col_trop <- "black"
tickl <- -0.4

quartz.options(height=5, width=7)
plot.new()
par(oma=c(1,1,1,1), mar=c(2,2,1,1), mfrow = c(2,2))

###### ---- 
plot.new()
plot.window(xlim=c(0,3), ylim=c(0,0.85))

rich1 <- sum(dff$rich[dff$state == 1])
rich2 <- sum(dff$rich[dff$state == 0])

wid <- 0.25
xv <- c(-wid, -wid, wid, wid)

polygon(x=1+xv, y = c(0, rich1, rich1, 0), border= col_trop, col=col_trop)
polygon(x=2+xv, y = c(0, rich2, rich2, 0), border= "black", col="gray90")

axis(1, at=c(-1,1,2,3), labels=c(NA, "Biome XX", "Biome YY", NA), cex.axis=0.9, tcl=tickl)
axis(2, at=seq(-0.2, 0.8, by=0.2), labels=F, tcl=tickl)
mtext("(a)", side=2, line=1.5, cex=1, at=0.85, las=1)

mtext("Species richness", side=2, line=1.5, cex=0.8)



######

plot.new()
plot.window(xlim=c(0,n+1), ylim=c(0,0.7))

colvec <- rep(col_trop, n)
#colvec[dff$state == 0 ] <- col_temp
dff <- dff[order(dff$rich, decreasing=T), ]
bgvec <- colvec
bgvec[dff$state == 0] <- "gray90"

points(1:n, dff$rich, pch = 21, bg=bgvec, col=colvec, cex=1.3)

axis(1, at = seq(-2, 26, by=2), labels = F, tcl=tickl)
axis(2, at= seq(-0.2,0.8, by=0.2), labels=F, tcl=tickl)

mtext("Species richness", side=2, line=1.5, cex=0.8)
mtext("Clade (ranked)", side=1, line= 1, cex=0.8)
mtext("(b)", side=2, line=1.5, cex=1, at=0.7, las=1)


#### 
######
 

plot.new()
plot.window(xlim=c(0,n+1), ylim=c(0,0.5))

temprate <- mean(dff$rate[dff$state == 0])
troprate <- mean(dff$rate[dff$state == 1])

dff <- dff[order(dff$rate, decreasing=F), ]
colvec <- rep(col_trop, n)
#colvec[dff$state == 0 ] <- col_temp
bgvec <- colvec
bgvec[dff$state == 0] <- "gray90"

lines(x=c(0,25), y=c(troprate, troprate), lwd=1, col="black")
lines(x=c(0,25), y=c(temprate, temprate), lwd=3, col="gray70", lty="dotted")

text(x=3.5, y = temprate, pos=3, label="Biome YY mean", cex=0.8, font=3)
text(x=20, y = troprate, pos=1, label="Biome XX mean", cex=0.8, font=3)


points(1:n, dff$rate, pch = 21, bg=bgvec, col=colvec, cex=1.3)

axis(1, at = seq(-2, 26, by=2), labels = F, tcl = tickl)
axis(2, at= seq(-0.1,0.6, by=0.1), labels=F, tcl = tickl)

mtext("Diversification rate", side=2, line=1.5, cex=0.8)
mtext("Clade (ranked)", side=1, line= 1, cex=0.8)
mtext("(c)", side=2, line=1.5, cex=1, at=0.5, las=1)

######

transparentColor <- function(namedColor, alpha = 0.8) {
	res <- col2rgb(namedColor) / 255;
	return(rgb(red = res[1,], green = res[2,], blue = res[3,], alpha = alpha));
}

c1 <- transparentColor("black")
c2 <- transparentColor("gray80")

d1 <- density(full_trop)
d2 <- density(full_temp)

plot.new()
plot.window(xlim=c(0, 0.6), ylim=c(0, 9))

polygon(d1$x, d1$y, col=c1)
polygon(d2$x, d2$y, col=c2)

axis(1, at = seq(-0.1,0.8, by=0.1), labels = F, tcl=tickl)
axis(2, at= seq(-2,10, by=2), labels=F, tcl = tickl)

mtext("Density", side=2, line=1.5, cex=0.8)
mtext("Diversification rate", side=1, line= 1, cex=0.8)
mtext("(d)", side=2, line=1.5, cex=1, at = 9, las=1)











