#######
#   Arguments:
#       phy     -  phylo object
#       p       -  plotting coordinates of phylogeny. returned invisibly from polar.phylo
#       node1   -  left bound tip node (may be character string)
#       node2   -  right bound tip node (may be character string)
#       col     -  fill and border color of polygon
#       alpha   -  transparency of color

colorClade = function(phy, p, node_mrca = NA, node1, node2, col, border, alpha) {
    
    n2 <- NA
    n1 <- NA
    
    if (!is.na(node_mrca)){
    	
    	tipset <- extract.clade(phy, node = node_mrca)$tip.label
    	indices <- match(tipset, phy$tip.label)
    	n2 <- max(indices)
    	n1 <- min(indices)
    	
    	
    }else{
    	if (mode(node1) == "character") {
        	node1 = match(node1, phy$tip.label)
    	}
    	if (mode(node2) == "character") {
        	node2 = match(node2, phy$tip.label)
    	}
    	n2 = max(node1, node2)
    	n1 = min(node1, node2)
    }
  
    
    seq.nod = .Call("seq_root2tip", phy$edge, phy$Nnode+1, phy$Nnode, PACKAGE="ape")
    nmrca = BAMMtools:::getmrca(phy, n1, n2)
    left = seq.nod[[n2]]
    left = left[which(left == nmrca):length(left)]
    right = seq.nod[[n1]]
    right = right[which(right == nmrca):length(right)]
    if (n1 == n2) {
        tips = seq.int(n1, n2-1)
    } else if (n1 == n2+1) {
        tips = seq.int(n1, n2-1)
    } else {
        tips = seq.int(n1, n2-1)
    }

    lc = p[rownames(p) %in% left,]
    rc = p[rownames(p) %in% right,]
    tc = p[rownames(p) %in% tips,]

    if (is.null(dim(tc))) {
        tc = matrix(tc, nrow = 1)
    }

    xv = c(zip(lc[-1,1],lc[-1,3]), tc[nrow(tc):1,3], zip(rc[nrow(rc):2,3], rc[nrow(rc):2,1]))
    yv = c(zip(lc[-1,2],lc[-1,4]), tc[nrow(tc):1,4], zip(rc[nrow(rc):2,4], rc[nrow(rc):2,2]))
    polygon(xv, yv, col=BAMMtools:::transparentColor(col, alpha), border=border)
}

#######
#   called by colorClade(...)
#
zip = function(x,y) {
    ret = numeric(2*length(x))
    i = 1 
    j =1
    while(i <= length(ret)) {
        ret[i] = x[j]
        i = i+1
        ret[i] = y[j]
        i = i+1
        j = j+1
    }
    return(ret)
}

########
# polar tree plotting function that returns coordinates of edges
#
#
polar.phylo = function(phy, vtheta=5, rbf=0.001, labels=FALSE, lwd=1, edge.color=1, arc.color=1)
{
    phy = BAMMtools:::getStartStopTimes(phy)
    tH = max(phy$end)
    rb = tH * rbf
    ret = BAMMtools:::setPolarTreeCoords(phy, vtheta, rbf)

    x0 = ret$segs[, 1]
    x1 = ret$segs[, 3]
    y0 = ret$segs[, 2]
    y1 = ret$segs[, 4]
    ofs = 0
    if (labels) {
        ofs = max(nchar(phy$tip.label) * 0.03 * cex)
    }

    plot.new()
    plot.window(xlim = c(-1, 1) + c(-rb, rb) + c(-ofs, ofs), ylim=c(-1, 1) + c(-rb, rb) + c(-ofs, ofs), asp=1)
    segments(x0, y0, x1, y1, col = edge.color, lwd = lwd, lend = 2)
    BAMMtools:::arc(0, 0, ret$arcs[,1], ret$arcs[,2], c(rb, rb + phy$end/tH), border=rep(arc.color, nrow(ret$arcs)), lwd=lwd)

    invisible(ret)
}


# v <- read.tree(...)
# z <- polar.phylo(...)
# colorClade(v, z, ...)





