####### ---

fitCRBD <- function(phy, nopt=5, lmin=0.001, lmax=5.0, MAXBAD = 200, sampling=1){
	
	if (length(phy$tip.label) < 3){
		pars <- c(0.0001,0)
		names(pars) <- c("lambda", "mu")
		return(pars)
	}
	
	fx <- make.bd(phy, sampling.f=sampling)
	
	for (i in 1:nopt){
	
		lam <- runif(1, 0, 0.5)	
	 	mu <- lam * runif(1, 0, 1)
	 
		badcount <- 0
 
		resx <- try(optim(c(lam, mu) ,fx, method='L-BFGS-B', control=list(maxit=1000, fnscale=-1), lower=lmin, upper=lmax), silent=T)
		while (class(resx) == 'try-error'){

			lam <- runif(1, 0, 0.5)	
	 		mu <- lam * runif(1, 0, 1)
			
			resx <- try(optim(c(lam, mu) , fx, method='L-BFGS-B', control=list(maxit=1000, fnscale=-1), lower=lmin, upper=lmax), silent=T);
			
			badcount <- badcount + 1;
			if (badcount > MAXBAD){
				stop("Too many fails in fitDiversitree\n")
			}
		}
		
		if (i == 1){
			best <- resx
		}else{
			if (best$value < resx$value){
				best <- resx
			}
		}
		
	}
	
	fres <- list(pars=best$par, loglik=best$value)
	fres$AIC <- -2*fres$loglik + 2*length(argnames(fx))
	fres$counts <- best$counts
	#fres$like_function <- fx
	fres$convergence <- best$convergence
	fres$message <- best$message
	 
	names(fres$pars) <- c("lambda", "mu")
	
	return(fres)
}



rfx <- function(x){
	
	rmat  <- matrix(NA, nrow=7, ncol=4)
	cvec <- c("Fw", "Br-Fw-Ma", "Fw-Ma", "Ma", "Br", "Br-Ma", "Br-Fw")
	for (i in 1:length(cvec)){
		tmp <- x[x$salinity == cvec[i], ]
		rmat[i, 1] <- nrow(tmp)
		if (nrow(tmp) > 0){
			rmat[i,2] <- mean(tmp$dr.mean)
			rmat[i,3] <- mean(tmp$tv.lambda)
			rmat[i,4] <- mean(tmp$tv.mu)
		}
	}
	
	res <- data.frame(salinity = cvec, rmat)
	colnames(res)[2:5] <- c("count", "dr", "lambda.bamm", "mu.bamm")
	return(res)
}

check_and_fix_ultrametric <- function(phy){
	
	if (!is.ultrametric(phy)){
		
		vv <- vcv.phylo(phy)
		dx <- diag(vv)
		mxx <- max(dx) - dx
		for (i in 1:length(mxx)){
			phy$edge.length[phy$edge[,2] == i] <- phy$edge.length[phy$edge[,2] == i] + mxx[i]
		}
		if (!is.ultrametric(phy)){
			stop("Ultrametric fix failed\n")
		}	
	}
	
	return(phy)
}

optimFx <- function(pars){
	pset <- exp(pars)
	ll   <- lfx_con(pset)
	cat(ll, "\n")
	return(ll)	
}



